/**
 * BSDialog4
 * @version v0.1.3.017 (2023/08/30 00:17)
 */
var BSDialog = {
	Show: async function (options) {
		const a = this;
		const _options = Object.assign(a.Default().ShowOptions, options);

		a.id = _options.ID;
		a.pfx = "bsdia4_";
		a.body = document.getElementsByTagName("body")[0];

		a.addBackdrop();
		a.addModal(_options.ID, _options.Title, _options.Size, true, _options.Colour);

		if (_options.URL == null) {
			await a.UpdateBody(_options.ID, _options.Message);
		} else {
			if (a.isNullOrWhitespace(_options.URL)) {
				await a.UpdateBody(_options.ID, _options.URL);
			} else {
				if (_options.URL.startsWith("http://") || _options.URL.startsWith("https://") || _options.URL.startsWith("/")) {
					await a.UpdateBodyRemote(_options.ID, _options.URL);
				} else {
					await a.UpdateBody(_options.ID, _options.URL);
				}
			}
		}
	},
	Prompt: async function (options) {
		const a = this;
		let id = "prompt" + Math.floor(Math.random() * 10000) + 1000;

		const _options = Object.assign(a.Default().PromptOptions, options);

		return await new Promise((resolve, reject) => {
			a.Create(id, _options.Title, _options.Message, _options.Size);

			let html = '';
			_options.Buttons.forEach(function(e) {
				html += '<button type="button" class="btn btn-' + e.Colour + '" data-prompt-value="' + e.Value + '">' + e.Label + '</button>';
			});

			a.UpdateFooter(id, html);

			const modal = a.Find(id);

			modal.Footer[0].querySelectorAll("button").forEach(function(button) {
				button.addEventListener("click", function(e){
					e.stopPropagation();
					e.preventDefault();

					const value = button.getAttribute("data-prompt-value");

					a.Close(id);

					resolve(value);
				});
			});

			modal.Close.forEach(function(sender) {
				sender.addEventListener("click", function(e){
					e.stopPropagation();
					e.preventDefault();

					a.Close(id);

					resolve("");
				});
			});

		});
	},
	Clear: function () {
		this.body.querySelectorAll(".modal").forEach(function(e) {
			e.parentNode.removeChild(e);
		});

		this.removeBackdrop();
	},
	Close: function (id) {
		let modal = this.Find(id);
		if (modal === null) {
		 	return;
		}

		modal.Modal.forEach(function(e) {
			e.parentNode.removeChild(e);
		});

		modal = this.Find(id);
		if (modal !== null) {
			return;
		}

		this.removeBackdrop();
	},
	Update: async function (options) {
		const a = this;
		let _options = Object.assign(a.Default().UpdateOptions, options);

		const modal = a.Find(_options.ID);
		if (modal === null) {
		 	return;
		}

		if (!this.isNullOrWhitespace(_options.Title)) {
			modal.Title.forEach(function(e) {
				e.innerHTML = _options.Title;
			});
		}

		if (!this.isNullOrWhitespace(_options.Body)) {
			modal.Body.forEach(function(e) {
				a.html(e, _options.Body);
			});
		}

		if (!this.isNullOrWhitespace(_options.BodyURL)) {
			if (_options.BodyURL.startsWith("http://") || _options.BodyURL.startsWith("https://") || _options.BodyURL.startsWith("/")) {
				// ok
			} else {
				_options.BodyURL = null;
			}
		}

		if (!this.isNullOrWhitespace(_options.BodyURL)) {
			await a.UpdateBodyRemote(_options.ID, _options.BodyURL);
		}

		if (!this.isNullOrWhitespace(_options.Footer)) {
			modal.Footer.forEach(function(e) {
				a.html(e, _options.Footer);
			});
		}

		if (!this.isNullOrWhitespace(_options.Size)) {
			modal.Modal[0].querySelectorAll(".modal-dialog").forEach(function(e) {

				e.classList.forEach(function(e2) {
					if (e2 == "modal-dialog") {
						return;
					}

					if (e2.startsWith("modal-")) {
						e.classList.remove(e2);
					}
				});

				e.classList.add("modal-" + _options.Size);
			});
		}
	},
	UpdateBodyRemote: async function (id, url) {
		var a = this;

		if (!a.Exists(id)) {
		 	return;
		}

		await fetch(url, {
			cache: 'no-cache',
			credentials: 'same-origin'
		}).then(data => data.text()).then(data => {
			a.UpdateBody(id, data);
		}).catch((error) => {
			a.UpdateBody(id, "Error: " + error);
		});
	},
	Exists: function (id) {
		return (this.Find(id) !== null);
	},
	Find: function (id) {
		let modal = this.body.querySelectorAll("#" + this.pfx + id + ".modal");
		if (!modal) {
			return null;
		}

		if (modal.length <= 0) {
			return null;
		}

		return {
			Title: modal[0].querySelectorAll(".modal-title"),
			Body: modal[0].querySelectorAll(".modal-body"),
			Footer: modal[0].querySelectorAll(".modal-footer"),
			Close: modal[0].querySelectorAll("[data-dismiss='modal']"),
			Modal: modal
		};
	},
	Default: function() {
		return {
			ShowOptions: {
				ID: null,
				Title: "",
				Message: "",
				URL: null,
				Size: "md",
				Colour: "secondary",
				ShowFooter: true
			},
			PromptOptions: {
				Title: "",
				Message: "",
				Size: "md",
				Buttons: [
					{ Label: "Yes", Value: "Yes", Colour: "primary" },
					{ Label: "No", Value: "No", Colour: "secondary" },
					{ Label: "Cancel", Value: "Cancel", Colour: "secondary" }
				]
			},
			ToastOptions: {
				ID: null,
				Title: "",
				Message: "",
				Size: "md"
			},
			UpdateOptions: {
				ID: null,
				Title: null,
				Body: null,
				BodyURL: null,
				Footer: null,
				Size: null
			}
		};
	},
	addBackdrop: function () {
		let a = this;

		// don't allow duplicates
		if (a.body.querySelectorAll(".modal-backdrop").length > 0) {
			return;
		}

		a.appendHtml(a.body, '<div class="modal-backdrop fade show"></div>');

		// lock background
		a.body.classList.add("modal-open");
		a.body.style.overflow = "hidden";

		// close
		let backdrop = a.body.querySelectorAll(".modal-backdrop")[0];
		backdrop.addEventListener("click", function(e){
			e.stopPropagation();
			e.preventDefault();

			a.Clear();
		});
	},
	addModal: function (id, title, size, showFooter, closeColour) {
		var a = this;

		// don't allow duplicates
		let modal = a.Find(id);
		if (modal !== null) {
		 	return;
		}

		let html = "";
		html += '<div class="modal fade show d-block" id="' + a.pfx + id + '" tabindex="-1">';
		html += '  <div class="modal-dialog modal-' + size + '">';
		html += '    <div class="modal-content">';
		html += '      <div class="modal-header">';
		html += '        <span class="modal-title font-weight-bold">' + title + '</span>';
		html += '        <button type="button" class="close" data-dismiss="modal" aria-label="Close">';
		html += '          <span aria-hidden="true">&times;</span>';
		html += '        </button>';
		html += '      </div>';
		html += '      <div class="modal-body">';

		html += '<div class="row mt-3 pb-3">';
		html += '  <div class="col-12 text-center">';
		html += '    <div class="spinner-border" role="status">';
		html += '	     <span class="sr-only">Loading...</span>';
		html += '    </div>';
		html += '  </div>';
		html += '</div>';

		html += '      </div>';

		if (showFooter === true) {
			html += '      <div class="modal-footer">';
			html += '        <button type="button" class="btn btn-' + closeColour + '" data-dismiss="modal">Close</button>';
			html += '      </div>';
		}

		html += '    </div>';
		html += '  </div>';
		html += '</div>';

		a.appendHtml(a.body, html);

		modal = a.Find(id);
		if (modal === null) {
			return;
	 	}

		modal.Close.forEach(function(e){
			e.addEventListener("click", function(e){
				e.stopPropagation();
				e.preventDefault();

		 		a.Close(id);
		 	});

			 e.addEventListener("auxclick", function(e){
				e.stopPropagation();
				e.preventDefault();

				if (e.button === 1) {
					a.toggleSize();
				}
		 	});

		});
	},
	appendHtml: function (el, html) {
		let node = document.createElement('template');
		node.innerHTML = html;

		node = node.content.firstChild;

		el.appendChild(node);
	},
	html: function (el, newHtml) {
		/// todo: replace with pure JS
		if (jQuery) {
			jQuery(el).html(newHtml);
		} else {
			el.innerHTML = newHtml;
		}
	},
	isNullOrWhitespace: function(e) {
		if (typeof (e) == "undefined") {
			return true;
		}

		if (e == null) {
			return true;
		}

		if (e == false) {
			return true;
		}

		return (e.trim().length <= 0);
	},
	removeBackdrop: function () {
		if (this.body.querySelectorAll(".modal-backdrop").length <= 0) {
			return;
		}

		if (this.body.querySelectorAll(".modal").length > 0) {
			return;
		}

		let backdrop = this.body.querySelectorAll(".modal-backdrop")[0];
		backdrop.parentNode.removeChild(backdrop);

		// unlock background
		this.body.classList.remove("modal-open");
		this.body.style.overflow = null;
	},
	toggleSize: function () {
		var a = this;

		let modal = a.Find(a.id);
		if (modal === null) {
		 	return;
		}

		let modalDialog = modal.Modal[0].querySelectorAll(".modal-dialog")[0];
		if (modalDialog.classList.contains('modal-sm')) {
			a.Update({ ID: a.id, Size: "md" });
		} else if (modalDialog.classList.contains('modal-md')) {
			a.Update({ ID: a.id, Size: "lg" });
		} else if (modalDialog.classList.contains('modal-lg')) {
			a.Update({ ID: a.id, Size: "xl" });
		} else if (modalDialog.classList.contains('modal-xl')) {
			a.Update({ ID: a.id, Size: "sm" });
		} else {
			a.Update({ ID: a.id, Size: "md" });
		}
	},
	//
	// deprecated/legacy-support
	Create: async function (id, title, url, size) {
		await this.Show({
			ID: id,
			Title: title,
			Message: url,
			URL: url,
			Size: size,
			Colour: "secondary"
		});
	},
	UpdateTitle: function (id, title) {
		this.Update({ ID: id, Title: title });
	},
	UpdateSize: function (id, size) {
		this.Update({ ID: id, Size: size });
	},
	UpdateBody: function (id, content) {
		this.Update({ ID: id, Body: content });
	},
	UpdateFooter: function (id, content) {
		this.Update({ ID: id, Footer: content });
	},
	Toast: async function (options) {
		const a = this;
		const _options = Object.assign(a.Default().ToastOptions, options);

		await this.Show({
			ID: _options.ID,
			Title: _options.Title,
			Message: _options.Message,
			URL: null,
			Size: _options.Size,
			Colour: "secondary",
			ShowFooter: false
		});
	},
	ShowToast: function (id, title, message, size) {
		this.Toast({ ID: id, Title: title, Message: message, Size: size });
	}
};